/** @mainpage libopencm3 EFM32 Giant Gecko

@version 1.0.0

@date 11 November 2012

API documentation for Energy Micro EFM32 Giant Gecko Cortex M3 series.

LGPL License Terms @ref lgpl_license
*/

/** @defgroup EFM32GG EFM32 Giant Gecko
Libraries for Energy Micro EFM32 Giant Gecko series.

@version 1.0.0

@date 11 November 2012

LGPL License Terms @ref lgpl_license
*/

/** @defgroup EFM32GG_defines EFM32 Giant Gecko Defines

@brief Defined Constants and Types for the Energy Micro EFM32 Giant Gecko series

@version 1.0.0

@date 11 November 2012

LGPL License Terms @ref lgpl_license
*/

