/** @mainpage libopencm3 STM32F0
 *
 * @version 1.0.0
 *
 * @date 11 July 2013
 *
 * API documentation for ST Microelectronics STM32F0 Cortex M0 series.
 *
 * LGPL License Terms @ref lgpl_license
*/

/** @defgroup STM32F0xx STM32F0xx
 * Libraries for ST Microelectronics STM32F0xx series.
 *
 * @version 1.0.0
 *
 * @date 11 July 2013
 *
 * LGPL License Terms @ref lgpl_license
 */

/** @defgroup STM32F0xx_defines STM32F0xx Defines
 *
 * @brief Defined Constants and Types for the STM32F0xx series
 *
 * @version 1.0.0
 *
 * @date 11 July 2013
 *
 * LGPL License Terms @ref lgpl_license
 */

